/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule activation-controller
 */

(function() {
var ActivationController;
var prototype;
var superConstructor;
var ActivationView = dixim.server.ActivationView;
var EventTarget = dixim.event.EventTarget;
var Formatter = dixim.ctt.util.Formatter;
var _ = dixim.ctt.ResourceBundle.get;
var ApiClientServer = dixim.common.ApiClientServer;
var BlockUi = dixim.common.BlockUi;
var UiFunctionManager = dixim.ctt.util.UiFunctionManager;

/**
 * @class ActivationController
 * @constructor
 * @param {Object} configs
 */
ActivationController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(ActivationController, EventTarget);
dixim.server.ActivationController = ActivationController;
prototype = ActivationController.prototype;
superConstructor = ActivationController.superConstructor;

var PREV = 'prev';

UiFunctionManager.set('dtcp_activation');

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    this._model = new ApiClientServer();
    this._model.addEventListener('fetchDas', this._successFetchDas, this);
    this._model.addEventListener('fetchDasError', this._errorFetchDas, this);
    this._model.addEventListener('activateDas', this._successActivateDas, this);
    this._model.addEventListener('activateDasError', this._errorActivateDas, this);

    this._view = new ActivationView(configs);
    this._view.addEventListener(PREV, this._clickPrev, this);

    this._parentContext = configs.parentContext;
    this._activationCount = 0;
    this._activationDate;
    this._blockUi = new BlockUi();
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
    this._fetchDas();
    this._view.setValue('auth_title', _('server_activation_auth_title'), 'text');
    this._view.setValue('auth_detail_title', _('server_activation_auth_detail_title'), 'text');
    this._view.setValue('auth_val', _('server_activation_auth_unknown'), 'val');
    this._view.setValue('auth_detail_val', _('server_activation_auth_detail_error_unknown'), 'text');
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._fetchDas = function() {
    this._blockUi.block();
    this._model.fetchDas();
};

prototype._activate = function() {
    this._activationCount += 1;
    this._activationDate = new Date();
    this._blockUi.block();
    this._model.activateDas();
};

prototype._successActivateDas = function(e) {
    this._blockUi.unblock();
    this._fetchDas();
};

prototype._errorActivateDas = function(e) {
    this._blockUi.unblock();
    this._fetchDas();
};

prototype._successFetchDas = function(e) {
    this._blockUi.unblock();
    this._updateDasStatus(e.args.status, e.args.errorCode);
};

prototype._errorFetchDas = function() {
    this._blockUi.unblock();
    this._view.setValue('auth_val', _('server_activation_auth_unknown'), 'val');
    this._view.setValue('auth_detail_val', _('server_activation_auth_summary_msg_fetch_fail'), 'text');
};

prototype._error = function(error_code) {
    var errorCodeStr;
    var errorCodeHex = error_code.toString(16).toUpperCase();
    var detail;

    errorCodeStr = ['0x', ('00000000' + errorCodeHex).slice(-8)].join('');
    switch (errorCodeStr) {
        case '0x00011001':
        case '0x00011002':
        case '0x00011003':
        case '0x00011005':
        case '0x00011006':
        case '0x00011007':
        case '0x00011FFF':
        case '0x00020000':
        case '0x00021001':
        case '0x00021002':
        case '0x00021003':
            detail = [' (', errorCodeStr, ')', '<br />', _('server_activation_auth_detail_common')].join('');
            break;
        case '0x00011004':
        case '0x00011008':
            detail = [' (', errorCodeStr, ')', '<br />', _('server_activation_auth_detail_network_error')].join('');
            break;
        case '0x00021FFF':
        case '0x00022FFF':
        case '0x00023FFF':
        case '0x0002F002':
        case '0x0002F103':
        case '0x0002FFFF':
            detail = [' (', errorCodeStr, ')', '<br />', _('server_activation_auth_detail_server_down')].join('');
            break;
        case '0x00022001':
        case '0x00022002':
        case '0x00022003':
        case '0x00022004':
            date_str = Formatter.stringifyDate(this._activationDate, 'YYYY/MM/DD hh:mm:ss');
            detail = [
                ' (', errorCodeStr, ')', '<br />',
                _('server_activation_auth_detail_limit'),
                _('server_activation_auth_detail_limit_code'),
                errorCodeStr,
                _('server_activation_auth_detail_limit_date'),
                date_str,
                _('server_activation_auth_detail_limit_productname'),
            ].join('');
            break;
        case '0x0002F001':
            detail = [' (', errorCodeStr, ')', '<br />', _('server_activation_auth_detail_server_updated')].join('');
            break;
        default:
            detail = [' (', errorCodeStr, ')', '<br />', _('server_activation_auth_detail_error_unknown')].join('');
            break;
    }
    detail = [_('server_activation_auth_summary_msg_fail'), '<br />', detail].join('');
    this._view.setValue('auth_val', _('server_activation_auth_fail'), 'val');
    this._view.setValue('auth_detail_val', detail, 'text');
};

prototype._updateDasStatus = function(status, errorCode) {
    if ((status === 0) && (this._activationCount === 0)) {
        this._activate();
    } else if (status === 0) {
        this._error(errorCode);
    } else if (status === 1) {
        this._view.setValue('auth_val', _('server_activation_auth_success'), 'val');
        this._view.setValue('auth_detail_val', _('server_activation_auth_summary_msg_success'), 'text');
    } else if (status === 2) {
        this._view.setValue('auth_val', _('server_activation_auth_activate_process'), 'val');
        this._view.setValue('auth_detail_val', _('server_activation_auth_summary_msg_activate_process'), 'text');
    }
};

prototype._clickPrev = function() {
    this.stop();
    this._parentContext.start();
};

}());
