/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule auto-delete-controller
 */

(function() {
var AutoDeleteController;
var prototype;
var superConstructor;
var AutoDeleteView = dixim.server.AutoDeleteView;
var EventTarget = dixim.event.EventTarget;
var _ = dixim.ctt.ResourceBundle.get;
var Formatter = dixim.ctt.util.Formatter;
var ApiClientAutoDelete = dixim.common.ApiClientAutoDelete;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var BlockUi = dixim.common.BlockUi;
var UiFunctionManager = dixim.ctt.util.UiFunctionManager;
var Utils = dixim.ctt.util.Utils;

/**
 * @class AutoDeleteController
 * @constructor
 * @param {Object} configs
 */
AutoDeleteController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(AutoDeleteController, EventTarget);
dixim.server.AutoDeleteController = AutoDeleteController;
prototype = AutoDeleteController.prototype;
superConstructor = AutoDeleteController.superConstructor;

UiFunctionManager.set('autoDelete');

var APPLY = 'apply';
var PREV = 'prev';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'auto-delete'].join('');

    this._model = new ApiClientAutoDelete();
    this._model.addEventListener('fetchAvailability', this._successFetchAvailability, this);
    this._model.addEventListener('fetchAvailabilityError', this._errorFetchAvailability, this);
    this._model.addEventListener('fetchThreshold', this._successFetchThreshold, this);
    this._model.addEventListener('fetchThresholdError', this._errorFetchThreshold, this);
    this._model.addEventListener('setAvailability', this._successSetAvailability, this);
    this._model.addEventListener('setAvailabilityError', this._errorSetAvailability, this);
    this._model.addEventListener('setThreshold', this._successSetThreshold, this);
    this._model.addEventListener('setThresholdError', this._errorSetThreshold, this);

    this._view = new AutoDeleteView(configs);
    this._view.addEventListener(APPLY, this._clickApply, this);
    this._view.addEventListener(PREV, this._clickPrev, this);

    this._blockUi = new BlockUi();
    this._settingsData;
    this._processInfo = {};

    this._parentContext = configs.parentContext;
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
    this._blockUi.block();
    this._processInfo.error = false;
    this._processInfo.len = 2;
    this._model.fetchAvailability();
    this._model.fetchThreshold();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._successFetchAvailability = function(e) {
    this._view.setAvailabilityValue(Math.floor(e.args.availability));
    this._processInfo.len -= 1;
    if (this._processInfo.len === 0) this._blockUi.unblock();
};

prototype._errorFetchAvailability = function(e) {
    this._processInfo.len -= 1;
    if (this._processInfo.len !== 0) return;

     this._blockUi.unblock();

    if (!!!this._processInfo.error) {
        this._processInfo.error = true;
        this._errorDialog(_('server_auto_delete_availability_fetch_error'));
    }
};

prototype._successFetchThreshold = function(e) {
    var free = e.args.freeSpaceThreshold / 1024 / 1024 / 1024 * 100;
    var gb = Math.floor(free) / 100;

    this._view.setThresholdLimitValue(Formatter.stringifySize(e.args.maxFreeSpaceThreshold, 0));
    this._view.setThresholdValue(gb);
    this._processInfo.len -= 1;
    if (this._processInfo.len === 0) this._blockUi.unblock();
};

prototype._errorFetchThreshold = function(e) {
    if (this._processInfo.len !== 0) return;

     this._blockUi.unblock();

    if (!!!this._processInfo.error) {
        this._processInfo.error = true;
        this._errorDialog(_('server_auto_delete_threshold_fetch_error'));
    }
};

prototype._setAvailability = function() {
    var query = {};

    query.availability = this._settingsData.availability;
    this._blockUi.block();
    this._model.setAvailability(query);
};

prototype._successSetAvailability = function() {
    this._blockUi.unblock();
    this._setThreshold();
};

prototype._errorSetAvailability = function() {
    this._blockUi.unblock();
    if (!!!this._processInfo.error) {
        this._processInfo.error = true;
        this._errorDialog(_('server_auto_delete_availability_set_error'));
    }
};

prototype._setThreshold = function() {
    var query = {};

    query.freeSpaceThreshold = parseInt(this._settingsData.threshold, 10) * 1024 * 1024 * 1024;
    this._blockUi.block();
    this._model.setThreshold(query);
};

prototype._successSetThreshold = function() {
    this._blockUi.unblock();
};

prototype._errorSetThreshold = function() {
    this._blockUi.unblock();
    if (!!!this._processInfo.error) {
        this._processInfo.error = true;
        this._errorDialog(_('server_auto_delete_threshold_set_error'));
    }
};

prototype._clickApply = function(e) {
    this._processInfo.error = false;

    if (e.args.threshold === '') {
        this._errorDialog(_('server_auto_delete_threshold_set_empty_error'));
        return;
    } else if (!!!Utils.test(e.args.threshold, /^[0-9]{1,4}$/)) {
        this._errorDialog(_('server_auto_delete_threshold_set_invalid_error'));
        return;
    }

    this._settingsData = e.args;
    this._setAvailability();
};

prototype._clickPrev = function() {
    this.stop();
    this._parentContext.start();
};

prototype._errorDialog = function(text) {
    var query = {};
    var dialog;

    query.id = this._id;
    query.title = _('server_menu_auto_delete');
    query.description = text;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};

}());
