/**
 * @module ctt
 * @submodule transfer-status-monitor
 */
(function() {
var Event = dixim.event.Event;
var Timer = dixim.Timer;
var EventTarget = dixim.event.EventTarget;
var ContentsTransferApiClient = dixim.ctt.api.ContentsTransferApiClient;

var TransferStatusMonitor;
var prototype;

/**
 * Reports transfer_status action results repeatedly
 * with the interval specified.
 *
 * @class TransferStatusMonitor
 * @extends EventTarget
 * @uses Event
 * @uses Timer
 * @uses ContentsTransferApiClient
 * @constructor
 * @param {Object} [configs]
 * @param {Number} [configs.interval=2000] interval in msec.
 * @param {ContentsTransferApiClient} [configs.model]
 */
TransferStatusMonitor = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(TransferStatusMonitor, EventTarget);
dixim.ctt.TransferStatusMonitor = TransferStatusMonitor;
prototype = TransferStatusMonitor.prototype;

/**
 * Triggered when the content transferring is in progress.
 *
 * @event transferring
 */
var TRANSFERRING = 'transferring';

/**
 * Triggered when the content transferring is being cancelled.
 *
 * @event canceling
 */
var CANCELING = 'canceling';

/**
 * Triggered when the transferring has been done.
 *
 * @event done
 */
var DONE = 'done';

/**
 * Triggered when the transfer status was fetched.
 *
 * @event fetch
 */
var FETCH = 'fetch';

/**
 * @method _initialize
 * @protected
 * @param {Object} [configs]
 * @param {Number} [configs.interval=2000] interval in msec.
 * @param {ContentsTransferApiClient} [configs.model]
 */
prototype._initialize = function(configs) {
  var model = null;
  var interval = NaN;

  if (!!configs) {
    model = configs.model;
    interval = configs.interval;
  }
  interval = interval || 2000;
  model = model || new ContentsTransferApiClient();

  this._model = model;
  this._timer = new Timer({interval: interval});
  this._session = null;
  this._state = null;

  this._timer.addEventListener('tick', this._didTick, this);
  this._model.addEventListener('transferStatus', this._didFetchTransferStatus, this);
  this._model.addEventListener('transferStatusError', this._didFailFetchTransferStatus, this);
};

prototype._didTick = function(e) {
  if (!!this._session) {
    return;
  }
  this._session = this._model.transferStatus();
};

prototype._isDone = function(data) {
  var records = data.results;
  var len = records.length;
  var i;
  var s;

  if (!len) return true;

  for (i = 0; i < len; i += 1) {
    s = records[i].status;
    if (s === 'transferring') return false;
    if (s === 'waiting') return false;
  }

  return true;
};

prototype._hasError = function(data) {
  var records = data.results;
  var len = records.length;
  var i;

  if (!len) return false;

  for (i = 0; i < len; i += 1) {
    if (records[i].status === 'error') return true;
  }
  return false;
};

prototype._hasWarning = function(data) {
  var records = data.results;
  var len = records.length;
  var i;

  if (!len) return false;

  for (i = 0; i < len; i += 1) {
    if (records[i].status === 'errorUnsupported') return true;
    if (records[i].status === 'errorCannotRemove') return true;
    if (records[i].status === 'errorNoSpace') return true;
  }
  return false;
};

prototype._hasCancelled = function(data) {
  var records = data.results;
  var len = records.length;
  var i;

  if (!len) return false;

  for (i = 0; i < len; i += 1) {
    if (records[i].status === 'cancelled') return true;
  }
  return false;
};

prototype._didFetchTransferStatus = function(e) {
  var args = e.args;
  var session = args.session;
  var data = args.data;
  var ea = {};

  if (this._session !== session) {
    return;
  }
  this._session = null;

  if (session.isCancelled()) {
    return;
  }

  ea.done = this._isDone(data);
  ea.error = this._hasError(data);
  ea.warning = this._hasWarning(data);
  ea.cancelled = this._hasCancelled(data);
  ea.data = data;

  this.dispatchEvent(new Event(FETCH, ea));

  if (ea.done) {
    if (this._state === DONE) return;
    this._state = DONE;
    this.dispatchEvent(new Event(DONE, ea));
    return;
  }

  if (ea.cancelled) {
    if (this._state === CANCELING) return;
    this._state = CANCELING;
    this.dispatchEvent(new Event(CANCELING, ea));
    return;
  }

  if (this._state !== TRANSFERRING) {
    this._state = TRANSFERRING;
    this.dispatchEvent(new Event(TRANSFERRING, ea));
    return;
  }
};

prototype._didFailFetchTransferStatus = function(e) {
  var args = e.args;
  var session = args.session;

  if (this._session !== session) {
    return;
  }
  this._session = null;
};

/**
 * @method start
 */
prototype.start = function() {
  this._timer.start();
};

/**
 * @method stop
 */
prototype.stop = function() {
  if (!!this._session) {
    this._session.cancel();
  }
  this._timer.stop();
};

}());
