/**
 * @module ctt
 * @submodule container-listing-view
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var TreeView = dixim.ctt.TreeView;
var _ = dixim.ctt.ResourceBundle.get;
var $ = dixim.ctt.jQuery;

var ContainerListingView;
var prototype;

var ELEMENT_ID_PREFIX = 'container-listing-view-';
var TEMPLATE_VIEW = ['<div class="container-listing-view">',
                       '<div class="container-listing-view-thead">',
                         _('ctt_container_listing_view_header_label'),
                       '</div>',
                       '<div class="container-listing-view-tree-list-wrapper"></div>',
                     '</div>'].join('');
var TEMPLATE_TREE_CONTAINER = '<div id="${groupId}" class="ctt-tree-view"></div>';
var TEMPLATE_TREE_HEADER = '<span class="group-header">${groupName}</span>';

/**
 * Container list view.
 *
 * @class ContainerListingView
 * @constructor
 * @param {Object} configs
 * @param {String} configs.container
 */
ContainerListingView = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(ContainerListingView, EventTarget);
dixim.ctt.ContainerListingView = ContainerListingView;
prototype = ContainerListingView.prototype;

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {String} configs.container
 * @param {Boolean} configs.selectionEnabled
 * @param {Boolean} configs.embedded
 */
prototype._initialize = function(configs) {
  this._containerNode = configs.containerNode;
  this._selectionEnabled = !!configs.selectionEnabled;
  this._embedded = !!configs.embedded;
  this._treeViews = {};
  this._groups = {};
  this._didlList = {};
};

/**
 * An event that is sent when the container is selected.
 * 
 * @event selectContainer
 * @param {Event} event an Event object that contains the following properties in its <em>args</em> property.
 * @param {Object} event.args.container the selected container data object.
 * @param {Object} event.args.group the group data object that the selected container is contained.
 */
var SELECT_CONTAINER = 'selectContainer';

/**
 * An event that is sent when a server is selected.
 * 
 * @event selectServer
 * @param {Event} event an Event object that contains the following properties in its <em>args</em> property.
 * @param {Object} event.args.group the group data object describes the selected server.
 */
var SELECT_SERVER = 'selectServer';

/**
 * An event that is sent when the container is opened.
 * 
 * @event openContainer
 * @param {Event} event an Event object that contains the following properties in its <em>args</em> property.
 * @param {Object} event.args.container the container data object opened.
 * @param {Object} event.args.group the group data object that the opened container is contained.
 */
var OPEN_CONTAINER = 'openContainer';

/**
 * An event that is sent when the container is closed.
 *
 * @event closeContainer
 * @param {Event} event an Event object that contains the following properties in its <em>args</em> property.
 * @param {Object} event.args.container the container data object closed.
 * @param {Object} event.args.group the group data object that the closed container is contained.
 */
var CLOSE_CONTAINER = 'closeContainer';

/**
 * An event that is sent when a group is added.
 *
 * @event addGroup
 * @param {Event} event
 * @param {Object} event.args.group
 */
var ADD_GROUP = 'addGroup';

/**
 * An event that is sent when the view has been rendered.
 *
 * @event render
 * @param {Event} event
 */
var RENDER = 'render';

/**
 * @method updateGroups
 * @param {Object[]} groups {id: "id", rootContainerId: 'containerId', name: "name"}
 */
prototype.updateGroups = function(groups) {
  var len;
  var i;
  var group;

  this._clearAllGroups();

  len = groups.length;
  for (i = 0; i < len; i += 1) {
    group = groups[i];
    this._addGroup(group);
    this.dispatchEvent(new Event(ADD_GROUP, {group: group}));
  }
};

prototype._addGroup = function(group) {
  var id;
  var name;
  var node;
  var header;
  var tv;
  var escape = dixim.xml.escape;
  var encode = dixim.xml.encodeId;

  name = escape(group.name);
  id = ELEMENT_ID_PREFIX + encode(group.id);

  node = TEMPLATE_TREE_CONTAINER.replace('${groupId}', id);
  node = $(node).appendTo(this._treeListContainer);
  header = TEMPLATE_TREE_HEADER.replace('${groupName}', name);

  tv = new TreeView({containerNode: node, headerNode: $(header)});
  tv.addEventListener('clickHeader', this._didClickHeader, this);
  tv.addEventListener('clickNode', this._didClickNode, this);
  tv.addEventListener('toggle', this._didToggle, this);
  tv.show();

  this._treeViews[group.id] = tv;
  this._groups[group.id] = group;
};

prototype._clearAllGroups = function() {
  var i = null;
  var treeViews = this._treeViews;

  for (i in treeViews) {
    if (!treeViews.hasOwnProperty(i)) continue;
    treeViews[i].destroy();
  }
  this._treeListContainer.empty();
};

prototype._getGroupByTreeView = function(tv) {
  var i;
  var tvs = this._treeViews;

  for (i in tvs) {
    if (!tvs.hasOwnProperty(i)) continue;
    if (tvs[i] === tv) return this._groups[i];
  }
  return null;
};

prototype._getTreeViewByGroup = function(group) {
  return this._treeViews[group.id];
};

prototype._didClickHeader = function(e) {
  var event;
  var args = {};

  args.group = this._getGroupByTreeView(e.args.treeView);
  event = new Event(SELECT_SERVER, args);
  this.dispatchEvent(event);
};

/**
 * @method _didClickNode
 * @private
 */
prototype._didClickNode = function(e) {
  var event;
  var ea = e.args;
  var args = {};

  args.group = this._getGroupByTreeView(ea.treeView);
  args.container = this._didlList[ea.record.id];
  if (this._selectionEnabled) {
    this._selectContainer(args.group, args.container);
  }
  event = new Event(SELECT_CONTAINER, args);
  this.dispatchEvent(event);
};

prototype._clearSelection = function() {
  var i = null;
  var tvs = this._treeViews;

  for (i in tvs) {
    if (!tvs.hasOwnProperty(i)) continue;
    tvs[i].clearSelection();
  }
};

prototype._selectContainer = function(group, container) {
  var tv;

  this._clearSelection();
  tv = this._getTreeViewByGroup(group);
  tv.selectNode(container);
};

/**
 * @method _didToggle
 * @private
 */
prototype._didToggle = function(e) {
  var event;
  var ea = e.args;
  var args  = {};
  var type;

  type = e.args.opened ? OPEN_CONTAINER : CLOSE_CONTAINER;

  args.group = this._getGroupByTreeView(ea.treeView);
  args.container = this._didlList[ea.record.id];
  event = new Event(type, args);
  this.dispatchEvent(event);
};

prototype._isGroupRootContainerId = function(groupId, containerId) {
  var group = this._groups[groupId];

  if (!group) return false;
  if (group.rootContainerId === containerId) return true;
  return false;
};

prototype._addChildren = function(data, replace) {
  var nodeRecords = [];
  var tv;
  var didlList = this._didlList;
  var parentId = data.parentId;
  var groupId = data.groupId;

  if (!(tv = this._treeViews[groupId])) return;

  if (replace) {
    if (this._isGroupRootContainerId(groupId, parentId)) {
      tv.clear();
    } else {
      tv.removeChildren(parentId);
    }
  }

  dixim.array.each(data.children, function(i, child) {
    nodeRecords.push({
      id: child.id,
      label: child.title
    });
    didlList[child.id] = child;
  });

  if (this._isGroupRootContainerId(groupId, parentId)) {
    tv.addChildren(nodeRecords);
  } else {
    tv.addChildren(nodeRecords, parentId);
  }
};

/**
 * Add child elements to the parent specified.
 *
 * @method addChildren
 * @param {Object} data
 * @param {String} data.groupId
 * @param {String} data.parentId
 * @param {Object[]} data.children
 */
prototype.addChildren = function(data) {
  this._addChildren(data, false);
};

/**
 * Update view with the data specified.
 *
 * @method update
 * @param {Object} data
 * @param {String} data.groupId
 * @param {String} data.parentId
 * @param {Object[]} data.children
 */
prototype.update = function(data) {
  this._addChildren(data, true);
};

/**
 * @method _createView
 * @private
 */
prototype._createView = function() {
  this._view = $(TEMPLATE_VIEW).
               hide().appendTo($(this._containerNode));
  if (this._embedded) {
    $('.container-listing-view-thead', this._view).hide();
  }
  this._treeListContainer = $('.container-listing-view-tree-list-wrapper',
                              this._view);
  this.dispatchEvent(new Event(RENDER));
};

/**
 * @method show
 */
prototype.show = function() {
  if (!!!this._view) {
    this._createView();
  }
  this._view.show();
};

/**
 * @method hide
 */
prototype.hide = function() {
  this._view.hide();
};

/**
 * @method destroy
 */
prototype.destroy = function() {
  if (!!!this._view) return;

  this._clearAllGroups();
  this._view.remove();
  this._view = null;
};

/**
 * @method setBrowseError
 * @param {Object} group
 * @param {Object} container
 */
prototype.setBrowseError = function(group, container) {
  var tv;
  var errorData;

  errorData = {
    id: ['error-children-of-', container.id].join(''),
    label: _('ctt_container_listing_view_tree_node_browse_error'),
    type: 'error'
  };

  tv = this._treeViews[group.id];
  if (!!tv) {
    tv.removeChildren(container.id);
    tv.addChildren([errorData], container.id);
  }
};

}());
