/**
 * @module ctt
 * @submodule deletion-controller
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var DeletionView = dixim.ctt.DeletionView;
var ContentsTransferApiClient = dixim.ctt.api.ContentsTransferApiClient;
var _ = dixim.ctt.ResourceBundle.get;
var Decoder = dixim.ctt.Decoder;

var DeletionController;
var prototype;

/**
 * @class DeletionController
 * @constructor
 * @param {Object} configs
 * @param {Object} configs.targetItems
 * @param {ContentsTransferApiClient} [configs.model]
 */
DeletionController = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(DeletionController, EventTarget);
dixim.ctt.DeletionController = DeletionController;
prototype = DeletionController.prototype;

/**
 * Triggered when the deletion has been done.
 *
 * @event close
 * @param {Event} event
 * @param {Object} event.args
 * @param {Object} event.args.deleted true if the deletion is completed, otherwise false.
 */
var CLOSE = 'close';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {Object} configs.targetItems
 * @param {ContentsTransferApiClient} [configs.model]
 */
prototype._initialize = function(configs) {
  configs = configs || {};

  this._decoder = new Decoder();
  this._model = configs.model || new ContentsTransferApiClient();
  this._targetItems = configs.targetItems;
  this._view = new DeletionView({targetItems: this._targetItems});
  this._session = null;
  this._queue = [];
  this._deleted = false;
  this._deleting_id;
};

/**
 * @method start
 */
prototype.start = function() {
  var v = this._view;
  var m = this._model;

  v.addEventListener('delete', this._didClickDelete, this);
  v.addEventListener('cancel', this._didClickCancel, this);

  m.addEventListener('getToken', this._didGetToken, this);
  m.addEventListener('getTokenError', this._didFailGetToken, this);
  m.addEventListener('remove', this._didRemove, this);
  m.addEventListener('removeError', this._didFailRemove, this);

  v.show();
};

prototype._didClickDelete = function(e) {
  if (this._queue.length) return;

  this._view.showActivityIndicator();

  this._queue = this._targetItems.concat();
  this._deleted = true;
  this._processQueue();
};

prototype._processQueue = function() {
  var item;
  var total = this._targetItems.length;
  var queue = this._queue;

  if (!queue.length) {
    this._didFinishDeletion();
    return;
  }

  item = queue.shift();
  this._view.updateProgress(item, total - queue.length - 1, total);

  this._deleting_id = item.id;
  this.remove();
};

prototype.remove = function() {
  this._model.getToken();
};

prototype._didGetToken = function(e) {
  var token = e.args.data.token;
  var buf_arr = this._decoder.base64ToArrayBuffer(token);
  var r_buf_arr;
  var query = {};

  this._decoder.setData(buf_arr);
  this._decoder.decode();
  r_buf_arr = this._decoder.getData();

  query.id = this._deleting_id;
  query.token = this._decoder.base64FromArrayBuffer(r_buf_arr);
  this._session = this._model.remove(query);
};

prototype._didFailGetToken = function(e) {
  var errorCode = e.args.errorCode;
  var query = {};

  if (errorCode === 404) {
    query.id = this._deleting_id;
    this._session = this._model.remove(query);
  } else {
    this._view.hideActivityIndicator();
    this._session = null;
    this._queue = [];
    this._view.showError(_('ctt_deletion_view_error'));
  }
};

prototype._didRemove = function(e) {
  this._session = null;
  this._processQueue();
};

prototype._didFailRemove = function(e) {
  this._view.hideActivityIndicator();
  this._session = null;
  this._queue = [];
  this._view.showError(_('ctt_deletion_view_error'));
};

prototype._didClickCancel = function(e) {
  if (!this._queue.length && !!!this._session) {
    this._view.destroy();
    this.dispatchEvent(new Event(CLOSE, {deleted: this._deleted}));
    return;
  }
  this._queue = [];
};

prototype._didFinishDeletion = function(e) {
  this._view.hideActivityIndicator();
  this._view.destroy();
  this.dispatchEvent(new Event(CLOSE, {deleted: this._deleted}));
};

/**
 * @method stop
 */
prototype.stop = function() {
  this._view.hide();
};

/**
 * @method destroy
 */
prototype.destroy = function() {
  this._view.destroy();
};

}());
