/**
 * @module ctt
 * @submodule toolbar-view
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var Configs = dixim.ctt.Configs;
var $ = dixim.ctt.jQuery;

var ToolbarView;
var prototype;

/**
 * @class ToolbarView
 * @extends EventTarget
 * @constructor
 * @param {Object} configs
 */
ToolbarView = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(ToolbarView, EventTarget);
dixim.ctt.ToolbarView = ToolbarView;
prototype = ToolbarView.prototype;

/**
 * Triggered when the view has been rendered.
 *
 * @event render
 */
var RENDER = 'render';

var ELEMENT_ID_PREFIX = 'ctt-toolbar-view-';
var CLASS_TOOLBAR = 'ctt-toolbar';
var CLASS_TOOLBAR_BUTTON = 'ctt-toolbar-button';
var CLASS_TOOLBAR_BUTTON_HAS_CHILDREN = 'ctt-toolbar-button-has-children';
var CLASS_TOOLBAR_BUTTON_HAS_CHILDREN_OPENED = 'ctt-toolbar-button-has-children-opened';
var CLASS_TOOLBAR_BUTTON_RIGHT_ALIGNED = 'ctt-toolbar-button-right-aligned';
var TEMPLATE_TOOLBAR_CONTAINER = '<div/>';
var TEMPLATE_BUTTON_CONTAINER = '<ul/>';
var TEMPLATE_BUTTON = '<li id="${id}" class="ctt-toolbar-button"><a href="#">${icon}${label}</a></li>';
var TEMPLATE_BUTTON_ICON = '<img src="${icon}" align="absmiddle" class="ctt-toolbar-button-icon" />';

/**
 * @class Button
 * @for ToolbarView
 */
/**
 * ID of the button. When the button clicked,
 * Event.type is set to this value.
 *
 * @property id
 * @type {String}
 */
/**
 * Label of the button.
 *
 * @property label
 * @type {String}
 */
/**
 * Icon path of the button.
 *
 * This must be relative to the <em>imagePath</em> of Configs object.
 *
 * @property icon
 * @type {String}
 */
/**
 * Children of the button.
 * 
 * The children are shown as submenu of the button.
 * Children of children, (i.e. grand-children of the button)
 * is not supported and ignored.
 *
 * @property children
 * @type {Button[]}
 */

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {String} configs.containerNode
 * @param {Button[]} [configs.buttons]
 *
 *     {
 *       id: buttonId,
 *       label: _('ctt_toolbar_button_other'),
 *       icon: 'arrow_down.png',
 *       children: [
 *         {
 *           id: 'children-1',
 *           label: 'children 1'
 *         },
 *         {
 *           id: 'children-2',
 *           label: 'children 2'
 *         },
 *         {
 *           id: 'children-3',
 *           label: 'children 3'
 *         }
 *       ]
 *    }
 *
 */
prototype._initialize = function(configs) {
  configs = configs || {};

  this._containerNode = configs.containerNode;
  this._buttons = configs.buttons || [];
  this._className = configs.className;
  this._buttonNodes = {};
  this._disabledButtons = {};
};

prototype._encodeId = function(id) {
  return ELEMENT_ID_PREFIX + id;
};

prototype._decodeId = function(id) {
  return id.replace(new RegExp('^' + ELEMENT_ID_PREFIX), '');
};

prototype._createButtonNode = function(data) {
  var imagePath;
  var icon = '';

  imagePath = Configs.get('imagePath').replace('/$', '');
  if (!!data.icon) {
    icon = TEMPLATE_BUTTON_ICON.
           replace('${icon}', [imagePath, data.icon].join('/'));
  }

  b = TEMPLATE_BUTTON.
      replace('${id}', this._encodeId(data.id)).
      replace('${label}', data.label).
      replace('${icon}', icon);

  return $(b);
};

/**
 * @method _createView
 * @private
 */
prototype._createView = function() {
  var view;
  var bc;
  var b;
  var data;
  var i;
  var len;
  var buttons = this._buttons;
  var className = this._className;

  if (className) {
    className = [className, CLASS_TOOLBAR].join(' ');
  } else {
    className = CLASS_TOOLBAR;
  }

  view = $(TEMPLATE_TOOLBAR_CONTAINER).
           addClass(className).hide().
           appendTo(this._containerNode);
  bc = $(TEMPLATE_BUTTON_CONTAINER).appendTo(view);

  len = buttons.length;
  for (i = 0; i < len; i += 1) {
    data = buttons[i];
    b = this._createButtonNode(data);

    if (typeof data.children !== 'undefined') {
      b.addClass(CLASS_TOOLBAR_BUTTON_HAS_CHILDREN);
      this._appendChildNodes(b, data.children);
    }
    if (data.align === 'right') {
      b.addClass(CLASS_TOOLBAR_BUTTON_RIGHT_ALIGNED);
    }
    if (data.cssClass) {
      b.addClass(data.cssClass);
    }

    b.appendTo(bc);
    this._buttonNodes[data.id] = b;
  }

  bc.bind('click.ToolBar', $.proxy(this._didClickButton, this));
  this._view = view;

  this.dispatchEvent(new Event(RENDER));
};

prototype._appendChildNodes = function(parentNode, children) {
  var len;
  var i;
  var data;
  var b;
  var container;

  container = $(TEMPLATE_BUTTON_CONTAINER).appendTo(parentNode);

  len = children.length;
  for (i = 0; i < len; i += 1) {
    data = children[i];
    b = this._createButtonNode(data);
    b.appendTo(container);
  }
};

prototype._didClickButton = function(e) {
  var target;
  var id;

  target = $(e.target);
  if (target.is('a') || target.is('img')) {
    target = target.closest('.' + CLASS_TOOLBAR_BUTTON, target.parent());
    e.preventDefault();
  }

  if (!target.length || !target.is('li')) {
    return;
  }

  id = this._decodeId(target.attr('id'));
  if (!!this._disabledButtons[id]) {
    return;
  }

  if (target.hasClass(CLASS_TOOLBAR_BUTTON_HAS_CHILDREN)) {
    target.toggleClass(CLASS_TOOLBAR_BUTTON_HAS_CHILDREN_OPENED);
    return;
  }

  this.dispatchEvent(new Event(id));
};

/**
 * @method show
 */
prototype.show = function() {
  if (!!!this._view) {
    this._createView();
  }
  this._view.show();
};

/**
 * @method hide
 */
prototype.hide = function() {
  this._view.hide();
};

/**
 * @method destroy
 */
prototype.destroy = function() {
  this._view.hide();
  this._view = null;
};

prototype._disableButton = function(id) {
  this._disabledButtons[id] = true;
};

prototype._enableButton = function(id) {
  delete this._disabledButtons[id];
};

prototype._getButtonNode = function(id) {
  return this._buttonNodes[id];
};

}());
