/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule server-boot-controller
 */

(function() {
var ServerBootController;
var prototype;
var superConstructor;
var ServerBootView = dixim.server.ServerBootView;
var EventTarget = dixim.event.EventTarget;
var _ = dixim.ctt.ResourceBundle.get;
var ApiClientServer = dixim.common.ApiClientServer;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var BlockUi = dixim.common.BlockUi;
var Enum = dixim.ctt.util.Enum;

/**
 * @class ServerBootController
 * @constructor
 * @param {Object} configs
 */
ServerBootController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(ServerBootController, EventTarget);
dixim.server.ServerBootController = ServerBootController;
prototype = ServerBootController.prototype;
superConstructor = ServerBootController.superConstructor;

var PREV = 'prev';
var STATUS = 'status';

var ServerStatus = Enum.fetch('SERVER_STATUS');

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'server-boot'].join('');

    this._model = new ApiClientServer();
    this._model.addEventListener('fetchDmsStatus', this._successFetchDmsStatus, this);
    this._model.addEventListener('fetchDmsStatusError', this._errorFetchDmsStatus, this);
    this._model.addEventListener('changeDmsStatus', this._successChangeDmsStatus, this);
    this._model.addEventListener('changeDmsStatusError', this._errorChangeDmsStatus, this);

    this._view = new ServerBootView(configs);
    this._view.addEventListener(PREV, this._clickPrev, this);
    this._view.addEventListener(STATUS, this._clickStatus, this);

    this._parentContext = configs.parentContext;
    this._servreStatus = ServerStatus.UNKNOWN;
    this._blockUi = new BlockUi();
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();

    this._view.setValue('status_title', _('server_boot_status_title'), 'text');
    this._view.setValue('status_val', _('server_boot_status_getting_info_process'), 'val');
    this._view.setValue('status_btn', _('server_boot_status_stop'), 'val');
    this._view.buttonStatus('status_btn', null);

    this._fetchDmsStatus();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._fetchDmsStatus = function() {
    this._blockUi.block();
    this._model.fetchDmsStatus();
};

prototype._changeStatus = function(data) {
    var query = data;

    this._blockUi.block();
    this._model.changeDmsStatus(data);
};

prototype._viewStarting = function() {
    this._view.buttonStatus('status_btn', false);
    this._view.setValue('status_val', _('server_boot_status_starting_process'), 'val');
    this._view.setValue('status_btn', _('server_boot_status_start'), 'val');
};

prototype._viewStarted = function() {
    this._view.buttonStatus('status_btn', false);
    this._view.setValue('status_val', _('server_boot_status_starting'), 'val');
    this._view.setValue('status_btn', _('server_boot_status_stop'), 'val');
};

prototype._viewStopping = function() {
    this._view.buttonStatus('status_btn', false);
    this._view.setValue('status_val', _('server_boot_status_stopping_process'), 'val');
    this._view.setValue('status_btn', _('server_boot_status_stop'), 'val');
};

prototype._viewStopped = function() {
    this._view.buttonStatus('status_btn', false);
    this._view.setValue('status_val', _('server_boot_status_stopping'), 'val');
    this._view.setValue('status_btn', _('server_boot_status_start'), 'val');
};

prototype._viewStoppingService = function() {
    this._view.buttonStatus('status_btn', null);
    this._view.setValue('status_val', _('server_boot_status_service_stopping'), 'val');
};

prototype._successFetchDmsStatus = function(e) {
    var status = e.args.status;
    var viewOperation;
    
    if (status === 'start') {
        this._servreStatus = ServerStatus.STARTED;
        viewOperation = dixim.proxy(this._viewStarted, this);
    } else if (status === 'stop') {
        this._servreStatus = ServerStatus.STOPPED;
        viewOperation = dixim.proxy(this._viewStopped, this);
    } else {
        this._servreStatus = ServerStatus.STOPPING_SERVICE;
        viewOperation = dixim.proxy(this._viewStoppingService, this);
    }
    viewOperation();
    this._blockUi.unblock();
};

prototype._errorFetchDmsStatus = function(e) {
    this._viewStoppingService();
    this._blockUi.unblock();
};

prototype._successChangeDmsStatus = function(e) {
    switch(this._servreStatus) {
        case ServerStatus.STARTED:
            this._viewStopped();
            this._servreStatus = ServerStatus.STOPPED;
            break;
        case ServerStatus.STOPPED:
            this._viewStarted();
            this._servreStatus = ServerStatus.STARTED;
            break;
    }
    this._blockUi.unblock();
};

prototype._errorChangeDmsStatus = function(e) {
    var errorText;

    switch(this._servreStatus) {
        case ServerStatus.STARTED:
            errorText = _('server_boot_status_change_error', _('server_boot_status_stop'));
            break;
        case ServerStatus.STOPPED:
            errorText = _('server_boot_status_change_error', _('server_boot_status_start'));
            break;
    }
    this._blockUi.unblock();
    this._errorDialog(errorText);
    this._fetchDmsStatus();
};

prototype._clickPrev = function() {
    this.stop();
    this._parentContext.start();
};

prototype._clickStatus = function() {
    var data = {};

    if (this._servreStatus === ServerStatus.STARTED) {
        this._viewStopping();
        data.status = 'stop';
    } else if (this._servreStatus === ServerStatus.STOPPED) {
        this._viewStarting();
        data.status = 'start';
    }
    this._changeStatus(data);
};

prototype._errorDialog = function(text) {
    var query = {};
    var dialog;

    query.id = this._id;
    query.title = _('server_menu_boot_server');
    query.description = text;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};

}());
